/*
 * $Id: database.c,v 1.7 1995/05/30 20:52:17 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/stat.h>
#include "facesP.h"

#define DB_block_size 0
#define DB_flags      GDBM_READER
#define DB_mode       0
#define DB_fatal_func NULL

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void FaceInitDB(void)
#else
void FaceInitDB()
#endif
{
  FacesDB     *nextDB, *prevDB, *index;
  char         filename[MAXPATHLEN+1];
  char        *pathNames, *nextPath, *startPath;
  char        *fileNames, *nextFile, *startFile;
  struct stat  buf;

  /*
   * Check if this is the first time, if so do the real init.
   */
  if (  FacesPrivate.databaseFlags == 0)
    face_init();

  prevDB    = NULL;
  startPath = XtNewString(facesOptions.PathNames);

  /*
   * Find the first path in the path variable.
   */
  pathNames = startPath;
  nextPath  = strchr(startPath, ':');
  if (nextPath != NULL)
    *nextPath++ = 0;
  
  /*
   * Loop through all the pathnames.
   */
  while (pathNames != NULL) {

    /*
     * Get the first filename
     */
    startFile = XtNewString(facesOptions.DatabaseOrder);
    fileNames = startFile;
    nextFile  = strchr(startFile, ':');
    if (nextFile != NULL)
      *nextFile++ = 0;

    /*
     * Loop through all the filenames
     */
    while (fileNames != NULL) {
      /*
       * Add the pathname to the filename
       */
      strcpy(filename, pathNames);
      strcat(filename, "/");
      strcat(filename, fileNames);

      /*
       * If the file exists add it to the database list
       */
      if (stat(filename, &buf) != 0)
	strcat(filename, ".gdbm");
      if (stat(filename, &buf) == 0) {
	index = FacesPrivate.firstDB;
	while ((index != NULL) && (strcmp(index->path, filename) != 0))
	  index = index->next;
	if (index == NULL) {
	  nextDB       = XtNew(FacesDB);
	  nextDB->path = XtNewString(filename);
	  nextDB->dbf  = NULL;
	  nextDB->next = NULL;
	
	  if (FacesPrivate.firstDB == NULL)
	    FacesPrivate.firstDB = nextDB;
	  else
	    prevDB->next = nextDB;
	  prevDB = nextDB;
	}
      }

      /*
       * Get the next filename.
       */
      fileNames = nextFile;
      if (fileNames != NULL) {
	nextFile  = strchr(fileNames, ':');
	if (nextFile != NULL)
	  *nextFile++ = 0;
      }
    }

    /*
     * Get the next path name
     */
    XtFree(startFile);
    pathNames = nextPath;
    if (pathNames != NULL) {
      nextPath  = strchr(pathNames, ':');
      if (nextPath != NULL)
	*nextPath++ = 0;
    }
  }

  XtFree(startPath);

  /*
   * Set the flag that the database is initialised.
   */
  FacesPrivate.databaseFlags = 1;
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void face_openDB(FacesDB *index)
#else
void face_openDB(index)
  FacesDB *index;
#endif
{
  if (index->dbf == NULL)
    index->dbf  = gdbm_open(index->path, DB_block_size, DB_flags, DB_mode,
			    DB_fatal_func);
  
  if (index->dbf == NULL)
    WARNING (("Error opening %s", index->path));
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void FaceOpenDB(void)
#else
void FaceOpenDB()
#endif
{
  FacesDB *index;

  /*
   * Check to see if the database was already initialised
   */
  if (FacesPrivate.databaseFlags == 0)
    FaceInitDB();

  if (FacesPrivate.databaseFlags == 2)
    return;

  index = FacesPrivate.firstDB;

  while(index != NULL) {
    face_openDB(index);
    index = index->next;
  }

  /*
   * Set the flag to identify the database is open.
   */
  FacesPrivate.databaseFlags = 2;
}


/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void face_closeDB(FacesDB *index)
#else
void face_closeDB(index)
  FacesDB *index;
#endif
{
  if (index->dbf != NULL)
    gdbm_close(index->dbf);
  index->dbf = NULL;
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void FaceCloseDB(void)
#else
void FaceCloseDB()
#endif
{
  FacesDB *index;

  /*
   * See if the database is open.
   */
  if (FacesPrivate.databaseFlags != 2)
    return;

  /*
   * Close all the open databases
   */
  index = FacesPrivate.firstDB;

  while(index != NULL) {
    face_closeDB(index);
    index = index->next;
  }

  /*
   * Set the flag that the database is closed
   */
  FacesPrivate.databaseFlags = 1;
}
