/*
 * $Id: faces.h,v 1.7 1995/05/27 17:01:14 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FACES_H
#define FACES_H

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*************************************************************************
 * Structure to hold the faces, each face only appears once.
 *************************************************************************/
typedef struct Face {
  char         *name;
  struct Face  *iconPtr;
  Pixmap        icon, image, shape;
  int           width, height, icon_width, icon_height;
  int           counter;
  int           unknown;
  struct Face  *next, *prev;
} Face;

/*************************************************************************
 * Structure to hold all the user variables for the faces lib
 *************************************************************************/
typedef struct FacesOptions {
  Pixel         Background;
  Pixel         Foreground;
  char         *DatabaseOrder;
  int           Debug;
  char         *ImageOrder;
  char         *PathNames;
  int           SearchFunction;
  char         *TmpFile;
  Boolean       UseMailcap;
  Boolean       UseColorIcon;
  Boolean       XfaceFirst;
} FacesOptions;

extern FacesOptions  facesOptions;

/*************************************************************************
 * The search algorithmes
 *  searchDepth   : This algorithm will search all posabilities in a
 *                  database before moving on.
 *  searchBreadth : This will search all databases before moving on to
 *                  the next posability.
 *  searchKnown   : This will search a database for all posabilities with
 *                  a username, then the next database, and then will
 *                  search all databases for the unknown posabilities.
 *************************************************************************/
#define searchDepth   1
#define searchBreadth 2
#define searchKnown   3

/*************************************************************************
 * Some global variables used in the library. These should be defined in
 *  the user program.
 *************************************************************************/
extern Display        *display;        /* Display to use                 */
extern Screen         *screen;         /* Screen used                    */
extern Window          window;         /* Window that can be used        */

/*************************************************************************
 * Some variables that are defined in the library and might be used by
 *  the user program.
 *************************************************************************/
extern Face           *firstFace;      /* List of faces                  */

/*************************************************************************
 * Prototypes
 *************************************************************************/

/*************************************************************************
 * Miscellaneous functions
 *************************************************************************/
#if NeedFunctionPrototypes
void  FaceInit           (void);
void  FaceClose          (void);
void  FaceRemoveOne      (Face *);
void  FaceRemoveAll      (void);
void  FaceDelete         (Face *);
#else
void  FaceInit           ();
void  FaceClose          ();
void  FaceRemoveOne      ();
void  FaceRemoveAll      ();
void  FaceDelete         ();
#endif

/*************************************************************************
 * Init, open and close the databases.
 *************************************************************************/
#if NeedFunctionPrototypes
void  FaceInitDB         (void);
void  FaceOpenDB         (void);
void  FaceCloseDB        (void);
#else
void  FaceInitDB         ();
void  FaceOpenDB         ();
void  FaceCloseDB        ();
#endif

/*************************************************************************
 * Search the databases for the requested face.
 *************************************************************************/
#if NeedFunctionPrototypes
Face *FaceSearchDB       (char *, char *);
#else
Face *FaceSearchDB       ();
#endif

#endif
