/*
 * $Id: facesP.h,v 1.11 1995/05/30 20:52:17 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FACESP_H
#define FACESP_H

#include <gdbm.h>
#include "faces.h"

/*************************************************************************
 * Structure to hold the databases that are used.
 *************************************************************************/
typedef struct FacesDB {
  char           *path;
  GDBM_FILE       dbf;
  struct FacesDB *next;
} FacesDB;

/*************************************************************************
 * Structure to hold the mailcap entries
 *************************************************************************/
typedef struct Mailcap {
  char           *Major;
  char           *Minor;
  char           *cmd;
  struct Mailcap *next;
} Mailcap;

/*************************************************************************
 * Structure to hold the global variables used in the library.
 *  These are private variables.
 *************************************************************************/
typedef struct FacesPrivateStruct {
  int             databaseFlags;       /* Wheter initialised and open    */
  int             depth;               /* Depth of the screen            */
  FacesDB        *firstDB;             /* List of databases to search    */
  GC              gc;                  /* Used to copy images            */
  Mailcap        *mailcap;             /* How to display things          */
} FacesPrivateStruct;

extern FacesPrivateStruct FacesPrivate;

/*************************************************************************
 * Macros for a warning and an error.
 *************************************************************************/
#define WARNING(mesg) {                                                \
  printf ("Faceslib (%s %d): ", __FILE__, __LINE__);                   \
  printf mesg;                                                         \
  printf (".\n");                                                      \
}

#define ERROR(mesg) {                                                  \
  printf ("Faceslib (%s %d): ", __FILE__, __LINE__);                   \
  printf mesg;                                                         \
  printf (".\n");                                                      \
  exit (-1);                                                           \
}

/*************************************************************************
 * Prototypes
 *************************************************************************/

/*************************************************************************
 * Opening and closing a single database.
 *************************************************************************/
#if NeedFunctionPrototypes
void  face_openDB      (FacesDB *);
void  face_closeDB     (FacesDB *);
#else
void  face_openDB      ();
void  face_closeDB     ();
#endif

/*************************************************************************
 * Loading the faces
 *************************************************************************/
#if NeedFunctionPrototypes
void  face_loadUnknown (Face **);
void  face_loadXbm     (Face **, char *, int);
#ifdef XPM
void  face_loadGif     (Face **, char *, int);
void  face_loadXpm     (Face **, char *, int);
#endif
#ifdef XFACE
void  face_loadXFace   (Face **,char *);
#endif
#else
void  face_loadUnknown ();
void  face_loadXbm     ();
#ifdef XPM
void  face_loadGif     ();
void  face_loadXpm     ();
#endif
#ifdef XFACE
void  face_loadXFace   ();
#endif
#endif

/*************************************************************************
 * Mailcap functions
 *************************************************************************/
#if NeedFunctionPrototypes
void  face_readMailcap (void);
void  face_useMailcap  (Face **, char *, int , char *, char *, Boolean);
#else
void  face_readMailcap ();
void  face_useMailcap  ();
#endif

/*************************************************************************
 * Misc functions
 *************************************************************************/
#if NeedFunctionPrototypes
void  face_addFace     (Face *);
void  face_init        (void);
#else
void  face_addFace     ();
void  face_init        ();
#endif

#endif
