/*
 * $Id: load_xface.c,v 1.8 1995/05/30 20:52:17 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef XFACE

#include <stdio.h>
#if SYSV
# include <string.h>
#else
# include <strings.h>
#endif
#include <compface.h>
#include "facesP.h"

/*
 * This shoule be large enough to hold the xface stuff.
 */
#define FACEBUFLEN 2048

/*
 * Local prototypes.
 */
#if NeedFunctionPrototypes
char face_switchBits(char);
#else
char face_switchBits();
#endif

/*************************************************************************
 * Convert an xface to a pixmap.
 *************************************************************************/
#if NeedFunctionPrototypes
void face_loadXFace(Face **facePtr, char *xface)
#else
void face_loadXFace(facePtr, xface)
  Face **facePtr;
  char   *xface;
#endif
{
  Face          *face;
  Pixmap         icon, image;
  char           fbuf[FACEBUFLEN], data[PIXELS], *ptr;
  int            i, j, val;
  char           lsb, msb;

  if (xface == NULL)
    return;

  face = *facePtr;

  /*
   * Uncompress the xface, to a list of hexnumbers.
   */
  strcpy(fbuf, xface);
  switch (uncompface(fbuf)) {
  case -2:
    WARNING (("(uncompface) internal error"));
    return;
    break;

  case -1:
    WARNING (("(uncompface) insufficient or invalid data"));
    return;
    break;

  case  1:
    WARNING (("(uncompface) excess data ignored"));
    break;

  }

  /*
   * Convert the hexnumbers to an icon.
   */
  ptr = fbuf;
  for (i=0; i<WIDTH; i++) {
    for (j=0; j<WORDSPERLINE; j++) {
      sscanf(ptr, "0x%x", &val);
      msb = face_switchBits((val & 0xff00) >> 8);
      lsb = face_switchBits((val & 0x00ff));
      data[(i*WORDSPERLINE+j)*2    ] = msb;
      data[(i*WORDSPERLINE+j)*2 + 1] = lsb;
      ptr += 7;
    }
    ptr += 1;
  }

  /*
   * Convert the data to an face.
   */
  icon  = XCreateBitmapFromData(display, window, data, WIDTH, HEIGHT);
  if (FacesPrivate.depth > 1) {
    image = XCreatePixmap(display, window, WIDTH, HEIGHT, FacesPrivate.depth);
    XCopyPlane(display, icon, image, FacesPrivate.gc,
	       0, 0, WIDTH, HEIGHT, 0, 0, 1);
  } else
    image = icon;

  if (face == NULL) {
    face          = XtNew(Face);
    *facePtr      = face;
    face->name    = NULL;
    face->iconPtr = NULL;
    face->width   = WIDTH;
    face->height  = HEIGHT;
    face->image   = image;
    face->shape   = None;
    face->counter = 1;
    face->unknown = False;
    face->next    = NULL;
    face->prev    = NULL;
  }
  face->icon_width  = WIDTH;
  face->icon_height = HEIGHT;
  face->icon        = icon;
  
}

/*************************************************************************
 * Need to switch the bits around.
 *************************************************************************/
#if NeedFunctionPrototypes
char face_switchBits(char input)
#else
char face_switchBits(input)
  char input;
#endif
{
  char output;
  int  i;

  output = 0;

  for (i=0; i<8; i++) {
    output = output << 1;
    if (input & 0x01)
      output++;
    input  = input >> 1;
  }

  return output;
}

#endif
