/*
 * $Id: load_xpm.c,v 1.7 1995/05/30 20:52:17 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef XPM

#include <stdio.h>
#if SYSV
# include <string.h>
#else
# include <strings.h>
#endif
#include "facesP.h"
#include <X11/xpm.h>

#if NeedFunctionPrototypes
void face_loadXpm(Face **facePtr, char *data, int length)
#else
void face_loadXpm(facePtr, data, length)
  Face **facePtr;
  char  *data;
  int    length;
#endif
{
  Pixmap           image, shape;
  int              xpmError;
  XpmAttributes    attributes;
  Face            *face;
  XpmColorSymbol  *noneColor;
  char            *buffer;

  /*
   * If this function gets called with face information we are just
   * interested in a xbm icon. So return without loading this one.
   */
  if (*facePtr != NULL)
    return;

  buffer = XtMalloc(length + 1);
  strncpy(buffer, data, length);
  *(buffer+length) = 0;

  face = NULL;
  attributes.valuemask = 0;

  /*
   * Make none eq background
   */
  noneColor = XtNew(XpmColorSymbol);
  noneColor->name  = "None";
  noneColor->value = NULL;
  noneColor->pixel = facesOptions.Background;
  attributes.colorsymbols = noneColor;
  attributes.numsymbols   = 1;
  attributes.closeness    = 10000;
  attributes.valuemask |= XpmColorSymbols | XpmCloseness;

  /*
   * Load the xpm file.
   */
  if (length > 0)
    xpmError = XpmCreatePixmapFromBuffer(display, window, buffer,
					 &image, &shape, &attributes);
  else
    xpmError = XpmReadFileToPixmap(display, window, data,
				   &image, &shape, &attributes);

  switch (xpmError) {
  case XpmColorError  :
    WARNING (("Color error while loading buffer"));
    break;

  case XpmSuccess     :
    face          = XtNew(Face);
    *facePtr      = face;
    face->name    = NULL;
    face->iconPtr = NULL;
    face->width   = attributes.width;
    face->height  = attributes.height;
    if (facesOptions.UseColorIcon) {
      face->icon_width  = attributes.width;
      face->icon_height = attributes.height;
      face->icon        = image;
    } else {
      face->icon_width  = 0;
      face->icon_height = 0;
      face->icon        = None;
    }
    face->image   = image;
    face->shape   = shape;
    face->counter = 1;
    face->unknown = False;
    face->next    = NULL;
    face->prev    = NULL;
    break;

  case XpmOpenFailed  :
    if (length > 0) {
      WARNING (("Could not open data ???"));
    } else {
      WARNING (("Could not open file %s", data));
    }
    break;

  case XpmFileInvalid :
    if (length > 0) {
      WARNING (("Data is invalid"));
    } else {
      WARNING (("Invalid file %s", data));
    }
    break;

  case XpmNoMemory    :
    if (length > 0) {
      WARNING (("Not enough memory to load data"));
    } else {
      WARNING (("Not enough memory to load file %s", data));
    }
    break;

  case XpmColorFailed :
    if (length > 0) {
      WARNING (("Color error while loading data"));
    } else {
      WARNING (("Color error while loading file %s", data));
    }
    break;

  default             :
    WARNING (("Never heard of this error %d", xpmError));
    break;
  }
  
  XtFree(buffer);
}

#endif
