/*
 * $Id: mailcap.c,v 1.9 1995/05/30 20:52:17 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/param.h>
#include "facesP.h"

/*
 * local prototypes
 */
#if NeedFunctionPrototypes
FILE *face_openMailcap (void);
char *face_writeTmpFile(char *, int);
#else
FILE *face_openMailcap ();
char *face_writeTmpFile();
#endif

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void face_readMailcap(void)
#else
void face_readMailcap()
#endif
{
  FILE    *fp;
  Mailcap *next, *prev;
  char     line[2048], *Major, *Minor, *cmd;

  prev = NULL;
  fp   = NULL;

  if (FacesPrivate.mailcap == NULL) {
    next        = XtNew(Mailcap);
    next->Major = XtNewString("image");
    next->Minor = XtNewString("xbm");
    next->cmd   = XtNewString("internal");
    next->next  = NULL;
    prev        = next;
    
    FacesPrivate.mailcap = next;

#ifdef XPM
    next        = XtNew(Mailcap);
    next->Major = XtNewString("image");
    next->Minor = XtNewString("xpm");
    next->cmd   = XtNewString("internal");
    next->next  = NULL;
    prev->next  = next;
    prev        = next;

    next        = XtNew(Mailcap);
    next->Major = XtNewString("image");
    next->Minor = XtNewString("gif");
    next->cmd   = XtNewString("internal");
    next->next  = NULL;
    prev->next  = next;
    prev        = next;
#endif
  } else {
    prev = FacesPrivate.mailcap;
    while (prev->next != NULL)
      prev = prev->next;
  }

  /*
   * Open a mailcap file
   */
  fp = face_openMailcap();

  /*
   * Now load the mailcap file.
   */
  while (!feof(fp)) {
    fgets(line, 2048, fp);
    if ((line[0] != '#') && (line[0] != '\n') && !feof(fp)) {
      line[strlen(line)-1] = 0;
      Major = line;
      Minor = strchr(Major, '/'); *Minor++ = 0;
      cmd   = strchr(Minor, ';'); *cmd++   = 0;
      while((*cmd == '\t') || (*cmd == ' '))
	cmd++;

      next        = XtNew(Mailcap);
      next->Major = XtNewString(Major);
      next->Minor = XtNewString(Minor);
      next->cmd   = XtNewString(cmd);
      next->next  = NULL;
      prev->next  = next;
      prev        = next;
    }
  }
  fclose(fp);
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
FILE *face_openMailcap(void)
#else
FILE *face_openMailcap()
#endif
{
  FILE *fp;
  char *env, filename[MAXPATHLEN+1];

  fp  = NULL;
  env = getenv("MAILCAP");
  if (env != NULL) {
    fp = fopen(env, "r");
  }
  if (fp == NULL) {
    env = getenv("HOME");
    if (env != NULL) {
      sprintf(filename, "%s/.mailcap", env);
      fp = fopen(filename, "r");
    }
  }
  if (fp == NULL) {
    strcpy(filename, "/usr/lib/mailcap");
    fp = fopen(filename, "r");
  }
  if (fp == NULL) {
    strcpy(filename, "/usr/local/lib/mailcap");
    fp = fopen(filename, "r");
  }

  return fp;
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void face_useMailcap(Face **face, char *data, int length, char *Major,
		     char *Minor, Boolean useEntry)
#else
void face_useMailcap(face, data, length, Major, Minor, useEntry)
  Face    **face;
  char     *data, *Major, *Minor;
  int       length;
  Boolean   useEntry;
#endif
{
  Mailcap *index;
  char     fmt[2048], cmd[2048], *filename;

  index    = FacesPrivate.mailcap;
  filename = NULL;

  while (index != NULL) {
    if (strcmp(index->Major, Major) == 0) {
      if ((strcmp(index->Minor, Minor) == 0) ||
	  (strcmp(index->Minor, "*") == 0)) {

	if (strcmp(index->cmd, "internal") == 0) {
#ifdef XPM
	  /*
	   * Create an face from an gif image
	   */
	  if ((strcmp(Minor, "gif") == 0) && useEntry)
	    face_loadGif(face, data, length);

	  /*
	   * Create an face from an xpm image
	   */
	  if ((strcmp(Minor, "xpm") == 0) && useEntry)
	    face_loadXpm(face, data, length);
#endif

	  /*
	   * Create an face from an xbm image
	   */
	  if ((strcmp(Minor, "xbm") == 0) && useEntry)
	    face_loadXbm(face, data, length);

	  return;

	} else {
	  /*
	   * Use the mailcap entry to show this data
	   */
	  if (facesOptions.UseMailcap) {
	    filename = face_writeTmpFile(data, length);
	    sprintf(fmt, "(%s ; rm %s)&", index->cmd, filename);
	    sprintf(cmd, fmt, filename);
	    system(cmd);
	  }
	  return;
	}
      }
    }
    index = index->next;
  }
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
char *face_writeTmpFile(char *data, int length)
#else
char *face_writeTmpFile(data, length)
  char *data;
  int   length;
#endif
{
  FILE        *fp;
  char         a, b, *filename;
  struct stat  buf;
 
  filename = XtMalloc(strlen(facesOptions.TmpFile)+4);
  
  for (a = 'a'; a <= 'z'; a++) {
    for (b = 'a'; b <= 'z'; b++) {
      sprintf(filename, "%s_%c%c", facesOptions.TmpFile, a, b);
      if (stat(filename, &buf) < 0) {
	if (errno != ENOENT) {
	  perror("Stat filename:");
	  exit(-1);
	}
	
	/*
	 * Write the information in a tmpfile
	 */
	if ((fp = fopen(filename, "wb")) == NULL) {
	  perror("Opening tmp file");
	  FaceCloseDB();
	  exit(-1);
	}
	if (fwrite(data, 1, length, fp) != length) {
	  perror("Writing tmp file");
	  fclose(fp);
	  FaceCloseDB();
	  exit(-1);
	}
	fclose(fp);

	return filename;
      }
    }
  }

  ERROR(("Could not create file name of %", facesOptions.TmpFile));
}
