/*
 * $Id: facesdb_extract.c,v 1.3 1995/03/19 17:16:14 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <sys/param.h>
#include "facesdb.h"

/*************************************************************************
 * 
 *************************************************************************/
#if NeedFunctionPrototypes
void extractEntry(char *dbase, char *email, char *image, char *Major,
		  char *Minor)
#else
void extractEntry(dbase, email, image, Major, Minor)
  char *dbase,*email, *image, *Major, *Minor;
#endif
{
  GDBM_FILE  dbf;
  datum      key, content;
  FILE      *fp;
  char       filename[MAXPATHLEN+1];
  Contents  *contents, *next;

  /*
   * Open the database to read it.
   */
  dbf = gdbm_open(dbase, block_size, GDBM_READER, mode, fatal_func);
  if (dbf == NULL) {
    if (!noTxt)
      fprintf(stderr, "Sorry, could not open database %s.\n", dbase);
    return;
  }

  /*
   * Get the correct key from the database.
   */
  key.dptr  = XtNewString(email);
  key.dsize = strlen(email);
  content   = gdbm_fetch(dbf, key);
  if (content.dptr == NULL) {
    gdbm_close(dbf);
    XtFree(key.dptr);
    return;
  }

  /*
   * If it is a link just say it and return.
   */
  if (strncmp(content.dptr, "link:", 5) == 0) {
    if (!noTxt)
      fprintf(stderr, "%s is linked to %s\n", email, (content.dptr+5));
    XtFree(key.dptr);
    free(content.dptr);
    return;
  }

  /*
   * Get all the info from the entry and save the requested pieces.
   */
  contents = extractContents(&content);
  while (contents != NULL) {
    if (((Major == NULL) || (strcmp(Major, contents->Major) == 0)) &&
	((Minor == NULL) || (strcmp(Minor, contents->Minor) == 0))) {
      /*
       * Create a filename.
       */
      if (image != NULL)
	strcpy(filename, image);
      else
	strcpy(filename, "face");
      strcat(filename, ".");
      strcat(filename, contents->Minor);
      /*
       * Save the info to the file.
       */
      if ((fp = fopen(filename, "wb")) == NULL)
	perror("Opening file");
      else {
	if (fwrite(contents->Data, 1, contents->Length, fp) !=
	                                                    contents->Length)
	  perror("Writing file");
	fclose(fp);
      }
    }
    /*
     * Go to the next entry.
     */
    next = contents->next;
    XtFree((char *)contents);
    contents = next;
  }

  /*
   * Close the database and return memory.
   */
  XtFree(key.dptr);
  free(content.dptr);
  gdbm_close(dbf);
}
