/*
 * $Id: facesdb_list.c,v 1.2 1995/03/19 17:16:14 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "facesdb.h"
#include <stdlib.h>

/*************************************************************************
 * 
 *************************************************************************/
#if NeedFunctionPrototypes
void listEntry(char *dbase, char *email)
#else
void listEntry(dbase, email)
  char *dbase, *email;
#endif
{
  GDBM_FILE  dbf;
  datum      next, key, content;
  Contents  *contents, *index;
  Boolean    specific;
  char      *tmp;

  /*
   * Looking for a specific image?
   */
  if (email == NULL)
    specific = False;
  else
    specific = True;
  
  /*
   * Open the database to read it.
   */
  dbf = gdbm_open(dbase, block_size, GDBM_READER, mode, fatal_func);
  if (dbf == NULL) {
    if (!noTxt)
      fprintf(stderr, "Sorry, could not open database %s.\n", dbase);
    return;
  }

  /*
   * Get the correct key from the database.
   */
  if (specific) {
    key.dptr  = XtNewString(email);
    key.dsize = strlen(email);
  } else
    key = gdbm_firstkey(dbf);

  /*
   * Read all the faces from the database.
   */
  while (key.dptr != NULL) {
    content = gdbm_fetch(dbf, key);
    if (content.dptr != NULL) {
      /*
       * Get the name with an \0
       */
      tmp = XtMalloc(key.dsize + 1);
      strcpy(tmp, key.dptr);
      tmp[key.dsize] = '\0';
      if (!noTxt)
        printf("Found info for %s\n", tmp);
      XtFree(tmp);

      /*
       * Either a link or actual data.
       */
      if (strncmp(content.dptr, "link:", 5) == 0) {
	if (!noTxt)
          printf("It is a link to %s\n", (content.dptr + 5));
      } else {
	contents = extractContents(&content);
	/*
	 * Print the info.
	 */
	while(contents != NULL) {
	  if (!noTxt)
            printf ("%s/%s;\t", contents->Major, contents->Minor);
	  if (!noTxt && (contents->Length > 0))
	    printf ("Length of data %d.\n", contents->Length);
	  else
	    if (!noTxt)
              printf ("Image is %s\n", contents->Data);
	  index = contents->next;
	  XtFree((char *)contents);
	  contents = index;
	}
	free(content.dptr);
      }
    }

    /*
     * Find the next entry.
     */
    if (specific) {
      XtFree(key.dptr);
      key.dptr = NULL;
    } else {
      next = gdbm_nextkey(dbf, key);
      free(key.dptr);
      key.dptr  = next.dptr;
      key.dsize = next.dsize;
    }
  }

  /*
   * Close the database
   */
  gdbm_close(dbf);
}
