/*
 * $Id: facesdb_misc.c,v 1.4 1995/03/20 21:58:01 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "facesdb.h"
#include <stdlib.h>
#include <ctype.h>

/*************************************************************************
 * 
 *************************************************************************/
#if NeedFunctionPrototypes
Contents *extractContents(datum *content)
#else
Contents *extractContents(content)
  datum *content;
#endif
{
  datum     orig;
  Contents *result, *next, *prev;
  char     *ptr;

  result     = NULL;
  prev       = NULL;
  orig.dsize = content->dsize;
  orig.dptr  = content->dptr;
  
  while (orig.dsize > 0) {
    next         = XtNew(Contents);
    next->Major  = orig.dptr;
    next->Minor  = strchr(next->Major, '/'); *(next->Minor)++ = 0;
    ptr          = strchr(next->Minor, ':'); *ptr++           = 0;
    next->Data   = strchr(ptr, ':');         *(next->Data)++  = 0;
    next->Length = atol(ptr);
    next->next   = NULL;

    orig.dptr   = next->Data + next->Length;
    if (next->Length == 0)
      orig.dptr += strlen(next->Data) + 1;
    orig.dsize -= (orig.dptr - next->Major);

    if (result == NULL)
      result = next;
    else
      prev->next = next;
    
    prev = next;
  }

  return result;
}

/*************************************************************************
 * 
 *************************************************************************/
#if NeedFunctionPrototypes
Boolean checkAddress(char *dbase, char *address)
#else
Boolean checkAddress(dbase, address)
  char *dbase, *address;
#endif
{
  GDBM_FILE dbf;
  datum     key, content;

  /*
   * Open the database.
   */
  dbf = gdbm_open(dbase, block_size, GDBM_READER, mode, fatal_func);
  if (dbf == NULL)
    return FALSE;

  /*
   * Get the correct key from the database.
   */
  key.dptr  = XtNewString(address);
  key.dsize = strlen(address);
  content   = gdbm_fetch(dbf, key);

  /*
   * Close the database.
   */
  gdbm_close(dbf);

  /*
   * Return the result.
   */
  XtFree(key.dptr);
  if (content.dptr != NULL) {
    free(content.dptr);
    return True;
  }

  return False;
}

/*************************************************************************
 * 
 *************************************************************************/
#if NeedFunctionPrototypes
void lowerAddress(char *address)
#else
void lowerAddress(address)
  char *address;
#endif
{
  char *index;

  if (address == NULL)
    return;

  /*
   * Turn the address into lower case
   */
  index = address;
  while ((*index != 0) && (strcmp(index, "MISC") != 0)) {
    if (isupper(*index))
      *index = tolower(*index);
    index++;
  }
}

